import commands2
from util import ConditionHolder


def test_wait_until(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd = commands2.WaitUntilCommand(cond.getCondition)

    scheduler.schedule(cmd)
    scheduler.run()

    assert scheduler.isScheduled(cmd)
    cond.setTrue()

    scheduler.run()
    assert not scheduler.isScheduled(cmd)



# autogenerated by 'robotpy-build create-imports commands2.button commands2._impl.button'
from .._impl.button import (
    Button,
    CommandGenericHID,
    CommandJoystick,
    CommandPS4Controller,
    CommandXboxController,
    JoystickButton,
    NetworkButton,
    POVButton,
)

__all__ = [
    "Button",
    "CommandGenericHID",
    "CommandJoystick",
    "CommandPS4Controller",
    "CommandXboxController",
    "JoystickButton",
    "NetworkButton",
    "POVButton",
]



import commands2
import pytest


def test_multiple_groups():
    cmd1 = commands2.InstantCommand()
    cmd2 = commands2.InstantCommand()

    _ = commands2.ParallelCommandGroup([cmd1, cmd2])
    with pytest.raises(RuntimeError):
        commands2.ParallelCommandGroup(cmd1, cmd2)


def test_externally_scheduled(scheduler: commands2.CommandScheduler):
    cmd1 = commands2.InstantCommand()
    cmd2 = commands2.InstantCommand()

    _ = commands2.SequentialCommandGroup([cmd1, cmd2])
    with pytest.raises(
        RuntimeError,
        match="Illegal use of Command: Commands that have been composed may not be added to another composition or scheduled individually!",
    ):
        scheduler.schedule(cmd1)


def test_redecorated():
    cmd = commands2.InstantCommand()

    _ = cmd.withTimeout(10).withInterrupt(lambda: False)

    with pytest.raises(RuntimeError):
        cmd.withTimeout(10)

    cmd.setGrouped(False)
    _ = cmd.withTimeout(10)



import enum
import commands2
from util import ConditionHolder


def test_select_command_int(scheduler: commands2.CommandScheduler):
    c = ConditionHolder()

    def _assert_false():
        assert False

    cmd1 = commands2.RunCommand(_assert_false)
    cmd2 = commands2.RunCommand(c.setTrue)
    cmd3 = commands2.RunCommand(_assert_false)

    sc = commands2.SelectCommand(lambda: 2, [(1, cmd1), (2, cmd2), (3, cmd3)])

    scheduler.schedule(sc)
    scheduler.run()

    assert c.cond


def test_select_command_str(scheduler: commands2.CommandScheduler):
    c = ConditionHolder()

    def _assert_false():
        assert False

    cmd1 = commands2.RunCommand(_assert_false)
    cmd2 = commands2.RunCommand(c.setTrue)
    cmd3 = commands2.RunCommand(_assert_false)

    sc = commands2.SelectCommand(lambda: "2", [("1", cmd1), ("2", cmd2), ("3", cmd3)])

    scheduler.schedule(sc)
    scheduler.run()

    assert c.cond


def test_select_command_enum(scheduler: commands2.CommandScheduler):
    c = ConditionHolder()

    def _assert_false():
        assert False

    class Selector(enum.Enum):
        ONE = enum.auto()
        TWO = enum.auto()
        THREE = enum.auto()

    cmd1 = commands2.RunCommand(_assert_false)
    cmd2 = commands2.RunCommand(c.setTrue)
    cmd3 = commands2.RunCommand(_assert_false)

    sc = commands2.SelectCommand(
        lambda: Selector.TWO,
        [
            (Selector.ONE, cmd1),
            (Selector.TWO, cmd2),
            (Selector.THREE, cmd3),
        ],
    )

    scheduler.schedule(sc)
    scheduler.run()

    assert c.cond



from commands2 import StartEndCommand
from util import CommandTestHelper


class StartEnd:
    def __init__(self) -> None:
        self.counter = 0

    def start(self):
        self.counter += 1

    def end(self):
        self.counter += 1


def test_start_end():
    with CommandTestHelper() as helper:
        se = StartEnd()

        cmd = StartEndCommand(se.start, se.end, [])

        helper.scheduler.schedule(cmd)
        helper.scheduler.run()
        helper.scheduler.run()
        helper.scheduler.cancel(cmd)

        assert se.counter == 2



from util import ConditionHolder, ManualSimTime
import commands2


def test_with_timeout(scheduler: commands2.CommandScheduler):
    with ManualSimTime() as tm:
        cmd = commands2.WaitCommand(10)
        timeout = cmd.withTimeout(2)

        scheduler.schedule(timeout)
        scheduler.run()

        assert not scheduler.isScheduled(cmd)
        assert scheduler.isScheduled(timeout)

        tm.step(3)

        scheduler.run()
        assert not scheduler.isScheduled(timeout)


def test_with_interrupt(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd = commands2.WaitCommand(10)
    timeout = cmd.withInterrupt(cond.getCondition)

    scheduler.schedule(timeout)
    scheduler.run()

    assert not scheduler.isScheduled(cmd)
    assert scheduler.isScheduled(timeout)

    cond.setTrue()

    scheduler.run()
    assert not scheduler.isScheduled(cmd)


def test_before_starting(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd = commands2.InstantCommand().beforeStarting(cond.setTrue)

    scheduler.schedule(cmd)

    assert cond.getCondition()

    scheduler.run()
    scheduler.run()

    assert not scheduler.isScheduled(cmd)


def test_and_then_fn(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd = commands2.InstantCommand().andThen(cond.setTrue)

    scheduler.schedule(cmd)

    assert not cond.getCondition()

    scheduler.run()
    scheduler.run()

    assert not scheduler.isScheduled(cmd)
    assert cond.getCondition()


def test_and_then_commands(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd1 = commands2.InstantCommand()
    cmd2 = commands2.InstantCommand(cond.setTrue)

    scheduler.schedule(cmd1.andThen(cmd2))

    assert not cond.getCondition()

    scheduler.run()

    assert cond.getCondition()


def test_deadline_with(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    dictator = commands2.WaitUntilCommand(cond.getCondition)
    endsBefore = commands2.InstantCommand()
    endsAfter = commands2.WaitUntilCommand(lambda: False)

    group = dictator.deadlineWith(endsBefore, endsAfter)

    scheduler.schedule(group)
    scheduler.run()

    assert scheduler.isScheduled(group)

    cond.setTrue()
    scheduler.run()

    assert not scheduler.isScheduled(group)


def test_along_with(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder(False)

    cmd1 = commands2.WaitUntilCommand(cond.getCondition)
    cmd2 = commands2.InstantCommand()

    group = cmd1.alongWith(cmd2)

    scheduler.schedule(group)
    scheduler.run()

    assert scheduler.isScheduled(group)

    cond.setTrue()
    scheduler.run()

    assert not scheduler.isScheduled(group)


def test_race_with(scheduler: commands2.CommandScheduler):
    cmd1 = commands2.WaitUntilCommand(lambda: False)
    cmd2 = commands2.InstantCommand()

    group = cmd1.raceWith(cmd2)

    scheduler.schedule(group)
    scheduler.run()

    assert not scheduler.isScheduled(group)


def test_perpetually(scheduler: commands2.CommandScheduler):
    cmd = commands2.InstantCommand().perpetually()

    scheduler.schedule(cmd)

    scheduler.run()
    scheduler.run()

    assert scheduler.isScheduled(cmd)



import commands2
from wpilib.simulation import DriverStationSim, pauseTiming, resumeTiming, stepTiming

# from unittest.mock import MagicMock


class ManualSimTime:
    def __enter__(self) -> "ManualSimTime":
        pauseTiming()
        return self

    def __exit__(self, *args):
        resumeTiming()

    def step(self, delta: float):
        stepTiming(delta)


class CommandTestHelper:
    def __init__(self) -> None:
        self.scheduler = commands2.CommandScheduler.getInstance()

    def __enter__(self):
        commands2.CommandScheduler.resetInstance()
        DriverStationSim.setEnabled(True)
        return self

    def __exit__(self, *args):
        pass


class Counter:
    def __init__(self) -> None:
        self.value = 0

    def increment(self):
        self.value += 1


class ConditionHolder:
    def __init__(self, cond: bool = False) -> None:
        self.cond = cond

    def getCondition(self) -> bool:
        return self.cond

    def setTrue(self):
        self.cond = True


class TestSubsystem(commands2.SubsystemBase):
    pass
